# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Tokenizer"]

Tokenizer: TypeAlias = Literal["pre_tokenized_array", "word_v0", "word_v1"]
